"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const document_1 = require("./document");
const query_1 = require("./query");
const aggregate_1 = require("./aggregate");
/**
 * 集合模块，继承 Query 模块
 *
 */
class CollectionReference extends query_1.Query {
    /**
     * 初始化
     *
     * @internal
     *
     * @param db    - 数据库的引用
     * @param coll  - 集合名称
     */
    constructor(db, coll) {
        super(db, coll);
    }
    /**
     * 读取集合名字
     */
    get name() {
        return this._coll;
    }
    /**
     * 获取文档的引用
     *
     * @param docID - 文档 ID
     */
    doc(docID) {
        if (!docID) {
            throw new Error('docID cannot be empty');
        }
        return new document_1.DocumentReference(this._db, this._coll, docID);
    }
    /**
     * 添加一篇文档
     *
     * @param data - 数据
     */
    add(data, options) {
        const docRef = new document_1.DocumentReference(this._db, this._coll, undefined);
        return docRef.create(data, options);
    }
    aggregate(rawPipeline = []) {
        return new aggregate_1.default(this._db, this._coll, rawPipeline);
    }
    /**
     * 创建索引
     */
    async createIndex(keys, options) {
        const docRef = new document_1.DocumentReference(this._db, this._coll, undefined);
        return docRef.createIndex(keys, options);
    }
    /**
     * 删除索引
     */
    async dropIndex(index) {
        const docRef = new document_1.DocumentReference(this._db, this._coll, undefined);
        return docRef.dropIndex(index);
    }
    async listIndexes() {
        const docRef = new document_1.DocumentReference(this._db, this._coll, undefined);
        return docRef.listIndexes();
    }
}
exports.CollectionReference = CollectionReference;
