"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bson_1 = require("bson");
const constant_1 = require("./constant");
const index_1 = require("./geo/index");
const index_2 = require("./serverDate/index");
/**
 * 工具模块
 *
 */
class Util {
}
exports.Util = Util;
/**
 * 格式化后端返回的文档数据
 *
 * @param document - 后端文档数据
 */
Util.formatResDocumentData = (documents) => {
    return documents.map((document) => {
        return Util.formatField(document);
    });
};
/**
 * 格式化字段
 *
 * 主要是递归数组和对象，把地理位置和日期时间转换为js对象。
 *
 * @param document
 * @internal
 */
Util.formatField = (document) => {
    const keys = Object.keys(document);
    let protoField = {};
    // 数组递归的情况
    if (Array.isArray(document)) {
        protoField = [];
    }
    keys.forEach((key) => {
        const item = document[key];
        const type = Util.whichType(item);
        let realValue;
        switch (type) {
            case constant_1.FieldType.GeoPoint:
                realValue = new index_1.Point(item.coordinates[0], item.coordinates[1]);
                break;
            case constant_1.FieldType.GeoLineString:
                realValue = new index_1.LineString(item.coordinates.map((point) => new index_1.Point(point[0], point[1])));
                break;
            case constant_1.FieldType.GeoPolygon:
                realValue = new index_1.Polygon(item.coordinates.map((line) => new index_1.LineString(line.map(([lng, lat]) => new index_1.Point(lng, lat)))));
                break;
            case constant_1.FieldType.GeoMultiPoint:
                realValue = new index_1.MultiPoint(item.coordinates.map((point) => new index_1.Point(point[0], point[1])));
                break;
            case constant_1.FieldType.GeoMultiLineString:
                realValue = new index_1.MultiLineString(item.coordinates.map((line) => new index_1.LineString(line.map(([lng, lat]) => new index_1.Point(lng, lat)))));
                break;
            case constant_1.FieldType.GeoMultiPolygon:
                realValue = new index_1.MultiPolygon(item.coordinates.map((polygon) => new index_1.Polygon(polygon.map((line) => new index_1.LineString(line.map(([lng, lat]) => new index_1.Point(lng, lat)))))));
                break;
            case constant_1.FieldType.Timestamp:
                realValue = new Date(item.$timestamp * 1000);
                break;
            case constant_1.FieldType.Object:
            case constant_1.FieldType.Array:
                realValue = Util.formatField(item);
                break;
            case constant_1.FieldType.ServerDate:
                realValue = new Date(item.$date);
                break;
            case constant_1.FieldType.ObjectId:
                realValue = bson_1.EJSON.deserialize(item);
                break;
            case constant_1.FieldType.Binary:
                realValue = bson_1.EJSON.deserialize(item);
                break;
            default:
                realValue = item;
        }
        if (Array.isArray(protoField)) {
            protoField.push(realValue);
        }
        else {
            protoField[key] = realValue;
        }
    });
    return protoField;
};
/**
 * 查看数据类型
 *
 * @param obj
 */
Util.whichType = (obj) => {
    let type = Object.prototype.toString.call(obj).slice(8, -1);
    if (type === constant_1.FieldType.Timestamp) {
        return constant_1.FieldType.BsonDate;
    }
    if (type === constant_1.FieldType.Object) {
        if (obj instanceof index_1.Point) {
            return constant_1.FieldType.GeoPoint;
        }
        else if (obj instanceof Date) {
            return constant_1.FieldType.Timestamp;
        } /* else if (obj instanceof Command) {
          return FieldType.Command;
        } */
        else if (obj instanceof index_2.ServerDate) {
            return constant_1.FieldType.ServerDate;
        }
        else if (obj instanceof bson_1.ObjectId) {
            return constant_1.FieldType.ObjectId;
        }
        else if (obj instanceof bson_1.Binary) {
            return constant_1.FieldType.Binary;
        }
        if (obj.$timestamp) {
            type = constant_1.FieldType.Timestamp;
        }
        else if (obj.$date) {
            type = constant_1.FieldType.ServerDate;
        }
        else if (index_1.Point.validate(obj)) {
            type = constant_1.FieldType.GeoPoint;
        }
        else if (index_1.LineString.validate(obj)) {
            type = constant_1.FieldType.GeoLineString;
        }
        else if (index_1.Polygon.validate(obj)) {
            type = constant_1.FieldType.GeoPolygon;
        }
        else if (index_1.MultiPoint.validate(obj)) {
            type = constant_1.FieldType.GeoMultiPoint;
        }
        else if (index_1.MultiLineString.validate(obj)) {
            type = constant_1.FieldType.GeoMultiLineString;
        }
        else if (index_1.MultiPolygon.validate(obj)) {
            type = constant_1.FieldType.GeoMultiPolygon;
        }
        else if (obj.$oid) {
            type = constant_1.FieldType.ObjectId;
        }
        else if (obj.$binary) {
            type = constant_1.FieldType.Binary;
        }
    }
    return type;
};
