import { SYMBOL_GEO_LINE_STRING } from '../helper/symbol';
import { Point } from './point';
import { isArray, isNumber } from '../utils/type';
/**
 * 线段
 *
 * @author starkewang
 */
var LineString = /** @class */ (function () {
    /**
     * 初始化
     *
     * @param points    - GeoPoint
     */
    function LineString(points) {
        if (!isArray(points)) {
            throw new TypeError("\"points\" must be of type Point[]. Received type " + typeof points);
        }
        if (points.length < 2) {
            throw new Error('"points" must contain 2 points at least');
        }
        points.forEach(function (point) {
            if (!(point instanceof Point)) {
                throw new TypeError("\"points\" must be of type Point[]. Received type " + typeof point + "[]");
            }
        });
        this.points = points;
    }
    LineString.prototype.parse = function (key) {
        var _a;
        return _a = {},
            _a[key] = {
                type: 'LineString',
                coordinates: this.points.map(function (point) { return point.toJSON().coordinates; }),
            },
            _a;
    };
    LineString.prototype.toJSON = function () {
        return {
            type: 'LineString',
            coordinates: this.points.map(function (point) { return point.toJSON().coordinates; }),
        };
    };
    LineString.validate = function (lineString) {
        if (lineString.type !== 'LineString' || !isArray(lineString.coordinates)) {
            return false;
        }
        for (var _i = 0, _a = lineString.coordinates; _i < _a.length; _i++) {
            var point = _a[_i];
            if (!isNumber(point[0]) || !isNumber(point[1])) {
                return false;
            }
        }
        return true;
    };
    LineString.isClosed = function (lineString) {
        var firstPoint = lineString.points[0];
        var lastPoint = lineString.points[lineString.points.length - 1];
        if (firstPoint.latitude === lastPoint.latitude &&
            firstPoint.longitude === lastPoint.longitude) {
            return true;
        }
    };
    Object.defineProperty(LineString.prototype, "_internalType", {
        get: function () {
            return SYMBOL_GEO_LINE_STRING;
        },
        enumerable: true,
        configurable: true
    });
    return LineString;
}());
export { LineString };
