import { SYMBOL_GEO_MULTI_LINE_STRING } from '../helper/symbol';
import { isArray, isNumber } from '../utils/type';
import { LineString } from './lineString';
/**
 * 多个 LineString
 *
 * @author starkewang
 */
var MultiLineString = /** @class */ (function () {
    /**
     * 初始化
     *
     * @param lines    - LineString
     */
    function MultiLineString(lines) {
        if (!isArray(lines)) {
            throw new TypeError("\"lines\" must be of type LineString[]. Received type " + typeof lines);
        }
        if (lines.length === 0) {
            throw new Error('Polygon must contain 1 linestring at least');
        }
        lines.forEach(function (line) {
            if (!(line instanceof LineString)) {
                throw new TypeError("\"lines\" must be of type LineString[]. Received type " + typeof line + "[]");
            }
        });
        this.lines = lines;
    }
    MultiLineString.prototype.parse = function (key) {
        var _a;
        return _a = {},
            _a[key] = {
                type: 'MultiLineString',
                coordinates: this.lines.map(function (line) {
                    return line.points.map(function (point) { return [point.longitude, point.latitude]; });
                }),
            },
            _a;
    };
    MultiLineString.prototype.toJSON = function () {
        return {
            type: 'MultiLineString',
            coordinates: this.lines.map(function (line) {
                return line.points.map(function (point) { return [point.longitude, point.latitude]; });
            }),
        };
    };
    MultiLineString.validate = function (multiLineString) {
        if (multiLineString.type !== 'MultiLineString' ||
            !isArray(multiLineString.coordinates)) {
            return false;
        }
        for (var _i = 0, _a = multiLineString.coordinates; _i < _a.length; _i++) {
            var line = _a[_i];
            for (var _b = 0, line_1 = line; _b < line_1.length; _b++) {
                var point = line_1[_b];
                if (!isNumber(point[0]) || !isNumber(point[1])) {
                    return false;
                }
            }
        }
        return true;
    };
    Object.defineProperty(MultiLineString.prototype, "_internalType", {
        get: function () {
            return SYMBOL_GEO_MULTI_LINE_STRING;
        },
        enumerable: true,
        configurable: true
    });
    return MultiLineString;
}());
export { MultiLineString };
