import { SYMBOL_GEO_MULTI_POINT } from '../helper/symbol';
import { Point } from './point';
import { isArray, isNumber } from '../utils/type';
/**
 * 多个 Point
 *
 * @author starkewang
 */
var MultiPoint = /** @class */ (function () {
    /**
     * 初始化
     *
     * @param points    - GeoPoint
     */
    function MultiPoint(points) {
        if (!isArray(points)) {
            throw new TypeError("\"points\" must be of type Point[]. Received type " + typeof points);
        }
        if (points.length === 0) {
            throw new Error('"points" must contain 1 point at least');
        }
        points.forEach(function (point) {
            if (!(point instanceof Point)) {
                throw new TypeError("\"points\" must be of type Point[]. Received type " + typeof point + "[]");
            }
        });
        this.points = points;
    }
    MultiPoint.prototype.parse = function (key) {
        var _a;
        return _a = {},
            _a[key] = {
                type: 'MultiPoint',
                coordinates: this.points.map(function (point) { return point.toJSON().coordinates; }),
            },
            _a;
    };
    MultiPoint.prototype.toJSON = function () {
        return {
            type: 'MultiPoint',
            coordinates: this.points.map(function (point) { return point.toJSON().coordinates; }),
        };
    };
    MultiPoint.validate = function (multiPoint) {
        if (multiPoint.type !== 'MultiPoint' || !isArray(multiPoint.coordinates)) {
            return false;
        }
        for (var _i = 0, _a = multiPoint.coordinates; _i < _a.length; _i++) {
            var point = _a[_i];
            if (!isNumber(point[0]) || !isNumber(point[1])) {
                return false;
            }
        }
        return true;
    };
    Object.defineProperty(MultiPoint.prototype, "_internalType", {
        get: function () {
            return SYMBOL_GEO_MULTI_POINT;
        },
        enumerable: true,
        configurable: true
    });
    return MultiPoint;
}());
export { MultiPoint };
