import { SYMBOL_GEO_POLYGON } from '../helper/symbol';
import { isArray, isNumber } from '../utils/type';
import { Polygon } from './polygon';
/**
 * 多个面
 *
 * @author starkewang
 */
var MultiPolygon = /** @class */ (function () {
    /**
     * 初始化
     *
     * @param polygons    - Polygon[]
     */
    function MultiPolygon(polygons) {
        if (!isArray(polygons)) {
            throw new TypeError("\"polygons\" must be of type Polygon[]. Received type " + typeof polygons);
        }
        if (polygons.length === 0) {
            throw new Error('MultiPolygon must contain 1 polygon at least');
        }
        for (var _i = 0, polygons_1 = polygons; _i < polygons_1.length; _i++) {
            var polygon = polygons_1[_i];
            if (!(polygon instanceof Polygon)) {
                throw new TypeError("\"polygon\" must be of type Polygon[]. Received type " + typeof polygon + "[]");
            }
        }
        this.polygons = polygons;
    }
    MultiPolygon.prototype.parse = function (key) {
        var _a;
        return _a = {},
            _a[key] = {
                type: 'MultiPolygon',
                coordinates: this.polygons.map(function (polygon) {
                    return polygon.lines.map(function (line) {
                        return line.points.map(function (point) { return [point.longitude, point.latitude]; });
                    });
                }),
            },
            _a;
    };
    MultiPolygon.prototype.toJSON = function () {
        return {
            type: 'MultiPolygon',
            coordinates: this.polygons.map(function (polygon) {
                return polygon.lines.map(function (line) {
                    return line.points.map(function (point) { return [point.longitude, point.latitude]; });
                });
            }),
        };
    };
    MultiPolygon.validate = function (multiPolygon) {
        if (multiPolygon.type !== 'MultiPolygon' ||
            !isArray(multiPolygon.coordinates)) {
            return false;
        }
        for (var _i = 0, _a = multiPolygon.coordinates; _i < _a.length; _i++) {
            var polygon = _a[_i];
            for (var _b = 0, polygon_1 = polygon; _b < polygon_1.length; _b++) {
                var line = polygon_1[_b];
                for (var _c = 0, line_1 = line; _c < line_1.length; _c++) {
                    var point = line_1[_c];
                    if (!isNumber(point[0]) || !isNumber(point[1])) {
                        return false;
                    }
                }
            }
        }
        return true;
    };
    Object.defineProperty(MultiPolygon.prototype, "_internalType", {
        get: function () {
            return SYMBOL_GEO_POLYGON;
        },
        enumerable: true,
        configurable: true
    });
    return MultiPolygon;
}());
export { MultiPolygon };
